H = load('GT/hotel/H.txt');

% load 'tracks' and 'groups' variables
load GT/hotel/annotation.mat
%load tracks_hotel_part.mat

% ============ sequential SFM ==============
walls = [];
tstep = 1;
deltat = tstep/25;
peddiam = 0.7;
tr = tracks(rem(tracks(:, 4), tstep) == 0, :);
[sfms_grmatrix pairs_conf] = sfm_seq_groups(tr, groups, walls, deltat, peddiam, []);

sfms_roc_fp = []; sfms_roc_fn = [];
sfms_thrs = [-10:0.5:-0.5];
for I=1:length(sfms_thrs)
  groups_sfms = find_groups(-sfms_grmatrix, sfms_thrs(I));
  [fp, fn] = group_compare(groups, groups_sfms);
  sfms_roc_fp(end+1) = size(fp, 1);
  sfms_roc_fn(end+1) = size(fn, 1);
end

% ============ Ge clustering ============
[gedists, getogether] = gedist(tracks);
ge_roc_fp = []; ge_roc_fn = [];
ge_thrs = [1,5,15, 20:20:200];
for I=1:length(ge_thrs)
  groups_ge = gecluster(gedists, getogether, ge_thrs(I));
  [fp, fn] = group_compare(groups, groups_ge);
  ge_roc_fp(end+1) = size(fp, 1);
  ge_roc_fn(end+1) = size(fn, 1);
end

% ============ Pellegrini - sequential ============
[pells_dists pairs_conf] = pelldist_seq(tracks, 50, []);
pells_dists = pells_dists / max(pells_dists(pells_dists < Inf));
pells_dists = 1 - pells_dists;
pells_dists(pells_dists == -Inf) = 0;
pells_roc_fp = []; pells_roc_fn = [];
pells_thrs = 0.3:0.1:1;
for I=1:length(pells_thrs)
  groups_pells = find_groups(pells_dists, pells_thrs(I));
  [fp, fn] = group_compare(groups, groups_pells);
  pells_roc_fp(end+1) = size(fp, 1);
  pells_roc_fn(end+1) = size(fn, 1);
end

% --------- plots ----------
figure;
plot(sfms_roc_fp, sfms_roc_fn, 'bo-', 'LineWidth', 2);
hold on
plot(pells_roc_fp, pells_roc_fn, 'r^-', 'LineWidth', 2);
plot(ge_roc_fp, ge_roc_fn, 'ks-', 'LineWidth', 2);
set(gca, 'FontSize', 15);
set(gcf, 'Position', [360   416   560   282]);
set(gcf, 'PaperPositionMode', 'auto');
xlabel('FP');
ylabel('FN');
xlim([0 100]);
ylim([0 40]);
title('HOTEL sequence, varying grouping distance threshold');
grid on
legend('SFM', 'd_P based A_{ij}', 'Ge at al. [7]');
