function [dists, together] = gedist(tracks)

  thr_s = 2;
  thr_v = 0.5;
  deltat = 1/25;
  alpha = 0.5;
  s_max = 5;
  v_max = 5;
  
  ids = sort(unique(tracks(:, 1)));

  % object track time interval
  frame_int = zeros(length(ids), 2);
  for I=1:length(ids)
    id_idxs = find(tracks(:, 1) == ids(I));
    frame_int(I, :) = [tracks(id_idxs(1), 4) tracks(id_idxs(end), 4)];
  end

  % distance from object I to object J
  dists = Inf * ones(length(ids), length(ids));
  together = zeros(length(ids), length(ids));
  
  for I=1:length(ids)
    % compute the velocity vectors
    trI = tracks(tracks(:, 1) == ids(I), :);
    trlen = size(trI, 1);
    vbeg = [ones(1, 10), 2:trlen-10];
    vend = 2:trlen;
    vlen = vend - vbeg;
    vI = (trI(vend, 2:3) - trI(vbeg, 2:3)) ./ ([vlen', vlen'] * deltat);
    vI = [vI(1, :); vI];	% velocity at the first frame the  same
                                % as in the second frame (to make the
                                % vectors the same length)
    
    for J=I+1:length(ids)
      ints = [frame_int(I, :) ids(I); frame_int(J, :) ids(J)];
      ints = sortrows(ints, 1);
      if ints(2, 1) < ints(1, 2)	% if overlap

	% compute the velocity vectors
	trJ = tracks(tracks(:, 1) == ids(J), :);
	trlen = size(trJ, 1);
	vbeg = [ones(1, 10), 2:trlen-10];
	vend = 2:trlen;
	vlen = vend - vbeg;
	vJ = (trJ(vend, 2:3) - trJ(vbeg, 2:3)) ./ ([vlen', vlen'] * deltat);
	vJ = [vJ(1, :); vJ];
	
	intA = ints(2, 1);
	intB = min(ints(:, 2));
	idxsI = find(trI(:, 4) >= intA & trI(:, 4) <= intB);
	idxsJ = find(trJ(:, 4) >= intA & trJ(:, 4) <= intB);
	sd = sqrt((trI(idxsI, 2) - trJ(idxsJ, 2)).^2 + ...
		  (trI(idxsI, 3) - trJ(idxsJ, 3)).^2);
	vd = sqrt((vI(idxsI, 1) - vJ(idxsJ, 1)).^2 + ...
		  (vI(idxsI, 2) - vJ(idxsJ, 2)).^2);

	together(I, J) = sum(sd < thr_s & vd < thr_v);
	
	% normalize sd and vd
%	sd(sd > s_max) = s_max;
%	sd = sd / s_max;
%	vd(vd > v_max) = v_max;
%	vd = vd / v_max;
	
	w = alpha * sd + (1-alpha) * vd;
	dists(I, J) = sum(w) / (together(I, J) * length(w));
      end
    end
  end
