function [grmatrix pairs_conf] = pelldist_seq(tracks, win_len, pairs)

  thr_standing = 0.15;			% m/s^2
  thr_together = 3;			% m
  deltat = 1/25;

  N = length(unique(tracks(:, 1)));

  win = zeros(N, N, win_len);
  win_pos = ones(N, N);
  
  grmatrix = -100000 * ones(N, N);
  
  first_frames = zeros(1, N);
  last_frames = zeros(1, N);
  TR = [];
  for I=1:N
    TR{I} = tracks(tracks(:, 1) == I, :);
    first_frames(I) = TR{I}(1, 4);
    last_frames(I) = TR{I}(end, 4);
  end
  frames = unique(tracks(:, 4));

  pairs_conf = [];
  for I=1:length(pairs)
    pairs_conf{I} = [];
  end
  
  for t=2:length(frames)
    disp(['Frame: ' num2str(t)]);

    tr_idxs = find(first_frames < frames(t-1) & last_frames > frames(t));

    if isempty(tr_idxs)
      continue;
    end

%    grmat = zeros(length(tr_idxs), length(tr_idxs));
    grmat = -100000 * ones(length(tr_idxs), length(tr_idxs));
    for I=1:length(tr_idxs)
      idx = tr_idxs(I);
      tr_posI = find(TR{idx}(:, 4) == frames(t));

      xI = TR{idx}(tr_posI, 2:3);

      vbeg = max(tr_posI-10, 1);
      vlen = tr_posI - vbeg;
      vI = (TR{idx}(tr_posI, 2:3) - TR{idx}(vbeg, 2:3))/(vlen*deltat);
      
      speedI = norm(vI);
				
      for J=1:length(tr_idxs)
	if I == J, continue; end
      
	idx = tr_idxs(J);
	tr_posJ = find(TR{idx}(:, 4) == frames(t));

	xJ = TR{idx}(tr_posJ, 2:3);

	vbeg = max(tr_posJ-10, 1);
	vlen = tr_posJ - vbeg;
	vJ = (TR{idx}(tr_posJ, 2:3) - TR{idx}(vbeg, 2:3))/(vlen*deltat);

	speedJ = norm(vJ);

	sd = norm(xI - xJ);
	Uij = xJ - xI;

	if sd > 3
	  continue;
	end

	d = 0;
	if speedI >= thr_standing & speedJ >= thr_standing
	  alpha = vecangle(vJ, Uij);
	  beta = vecangle(vI, vJ);
	  d = d + log(P_relative_position(sd, alpha)) + ...
	      log(P_group_angle_distance_moving(beta));
	else
	  d = d + log(P_group_distance_not_moving(sd)) + log(1/pi);
	end
	d = d - log(P_distance(sd)) - log(1/pi);
	grmat(I, J) = d;
      end
    end

    % window
    win_pos(tr_idxs, tr_idxs) = win_pos(tr_idxs, tr_idxs) + 1;
    win_pos(win_pos > win_len) = 1;
    
    [idxsA, idxsB] = ndgrid(tr_idxs, tr_idxs);
    idxsC = win_pos(tr_idxs, tr_idxs);
    idxs = sub2ind(size(win), idxsA(:), idxsB(:), idxsC(:));
    win(idxs) = grmat;

    cur_sum = sum(win(tr_idxs, tr_idxs, :), 3);
    grmatrix(tr_idxs, tr_idxs) = max(grmatrix(tr_idxs, tr_idxs), cur_sum);

    for K=1:size(pairs, 1)
      pos1 = find(tr_idxs == pairs(K, 1));
      pos2 = find(tr_idxs == pairs(K, 2));
      if ~isempty(pos1) & ~isempty(pos2)
	pairs_conf{K}(end+1) = cur_sum(pos1, pos2);
      end
    end
  end

  
function P = P_relative_position(dist, alpha) % Figure 4, top-left

  S = [0.05, 0; 0, pi/10];
  M = [0.6; pi/2];
  P = 1/(2*pi*(S(1,1)+S(2,2))) * ...
      exp(-0.5 * ([dist; alpha] - M)' * inv(S) * ([dist; alpha] - M));
  % normalise by integral of pdf over the region dist: 0-3; angle: 0-pi
  P = P / 0.9913024;

function P = P_group_angle_distance_moving(alpha) % Figure 4, top-right

  mu = -0.1; 
  sigma = 0.4;
  % normalisation to cutting it away at 0
  P = 1/sqrt(2*pi*sigma^2)*exp(-(alpha-mu).^2/(2*sigma^2)) / 0.4013;

function P = P_group_distance_not_moving(dist) % Figure 4, bottom-left

  a = 0.7; 
  mu1 = 0.85; 
  sigma1 = 0.21; 
  mu2 = 1.7; 
  sigma2 = 0.3; 
  P = a * 1/sqrt(2*pi*sigma1^2)*exp(-(dist-mu1).^2/(2*sigma1^2)) + ...
      (1-a) * 1/sqrt(2*pi*sigma2^2)*exp(-(dist-mu2).^2/(2*sigma2^2));

function P = P_distance(dist)		% Figure 4, bottom-right

  if dist < 0.3
    P = 0;
  else
    P = 0.2743 * (dist - 0.3);
  end

function P = P_orientation_change(alpha) % Figure 3, right
  lambda = 5;
  P = lambda * exp(-alpha * lambda);

function P = P_speed_prior(speed)	% Figure 3, left
  sigma = 0.3;
  mu = 0.75;
  lambda = 40;
  P = 0.5 * lambda * exp(-alpha*lambda) + ... % exponential distribution
      0.5 * 1/0.9938*1/sqrt(2*pi*sigma^2)*exp(-(alpha-mu).^2/(2*sigma^2)); % norm distr. + normalisation for triming it at zero
