function [GR grmatrix] = sfm_find_groups(ped, walls, deltat, vnew, pars)

  DEBUG = 0;
  
  N = length(ped);
  GR = zeros(1, N);
  prevGR = zeros(1, N);
  nextGRnum = 1;
  prev_err = Inf;
  grmatrix = zeros(N, N);
  
  [pednew F Frep] = sfm_predict(ped, GR, walls, deltat, deltat, pars);

  K = (vnew - reshape([ped.v], 2, N)') / deltat - F(:, [5 6]) - F(:, [9 10]);
  error = F(:, [11 12]) + F(:, [7 8]) - K;
  errors = sqrt(error(:, 1).^2 + error(:, 2).^2);

  [Y idxs] = sort(errors, 1, 'descend');
  queue = [Y, idxs];

  used = zeros(1, N);
  
  while ~isempty(queue)
    idxI = queue(1, 2);			% first pedestrian in the queue
    err = sum(errors);			% and the current prediction error

    if DEBUG
      err
    end
    
    % find the maximum repulsive force and the corresponding pedestrian
    [Frep_max idxJ] = max(squeeze(Frep(idxI, :, 1)).^2 + ...
			  squeeze(Frep(idxI, :, 2)).^2);

    if norm(ped(idxI).x - ped(idxJ).x) > 3 % 5m
      used(idxI) = 1;
      queue = queue(2:end, :);
      if DEBUG
	idxI
	idxJ
	disp('too far apart');
      end
      continue;
    end

    prevGR = GR;
    prevNextGRnum = nextGRnum;
    % assign pedestrians idxI and idxJ to the same group
    if GR(idxI) == 0 & GR(idxJ) == 0	% none is yet in a group
      GR(idxI) = nextGRnum;
      GR(idxJ) = nextGRnum;
      nextGRnum = nextGRnum + 1;
      grAssignType = 1;
    elseif GR(idxI) == 0 | GR(idxJ) == 0 % one is already part of a group
      GR(idxI) = max(GR(idxI), GR(idxJ));
      GR(idxJ) = max(GR(idxI), GR(idxJ));
      grAssignType = 2;
    else				% both are already in different groups
      mi = min(GR(idxI), GR(idxJ));
      ma = max(GR(idxI), GR(idxJ));
      GR(find(GR == ma)) = mi;
      a = find(GR > ma);
      GR(a) = GR(a) - 1;
      nextGRnum = nextGRnum - 1;
      grAssignType = 3;
    end

    % re-compute the errors
    prevFrep = Frep;
    prevF = F;
    prev_errors = errors;
    % do not attempt to group people walking in oposite direction
    if abs(vecangle(ped(idxI).v, ped(idxJ).v)) > pi/2
      errors(1:end) = Inf;
    else
      % those affected by the change in group labels - need to be
      % predicted by SFM again
      affected = find(GR == GR(idxI));
      
      [pednew F_part Frep_part] = sfm_predict(ped, GR, walls, deltat, ...
					      deltat, pars, ...
					      affected, ...
					      F(:, [5 6]) + F(:, [7 8]) + F(:, [9 10]) + F(:, [11 12]));
      F(affected, :) = F_part(affected, :);
      Frep(affected, :, :) = Frep_part(affected, :, :);

      K = (vnew - reshape([ped.v], 2, N)') / deltat - ...
	  F(:, [5 6]) - F(:, [9 10]);
      error = F(:, [11 12]) + F(:, [7 8]) - K;
      errors = sqrt(error(:, 1).^2 + error(:, 2).^2);
    end

    if DEBUG
      idxI
      idxJ
    end
    
    err_diff = err - sum(errors);
    if err_diff > 0
      [Y idxs] = sort(errors, 1, 'descend');
      queue = [Y, idxs];
      queue(used(idxs) == 1, :) = [];
      if DEBUG
	disp('accepted');
      end
      
      if grAssignType == 1
	grmatrix(idxI, idxJ) = err_diff;
	grmatrix(idxJ, idxI) = err_diff;
      elseif grAssignType == 2
	if prevGR(idxI) == 0
	  grmatrix(idxI, affected) = err_diff / (length(affected) - 1);
	  grmatrix(affected, idxI) = err_diff / (length(affected) - 1);
	else
	  grmatrix(idxJ, affected) = err_diff / (length(affected) - 1);
	  grmatrix(affected, idxJ) = err_diff / (length(affected) - 1);
	end
	grmatrix(idxI, idxI) = 0;
	grmatrix(idxJ, idxJ) = 0;
      else	% grAssignType == 3
	idxsA = find(prevGR == prevGR(idxI));
	idxsB = find(prevGR == prevGR(idxJ));
	for I=1:length(idxsA)
	  grmatrix(idxsA(I), idxsB) = ...
	      err_diff / (length(idxsA) * length(idxsB));
	  grmatrix(idxsB, idxsA(I)) = ...
	      err_diff / (length(idxsA) * length(idxsB));
	end
      end
    else
      % return to previous state
      used(idxI) = 1;
      GR = prevGR;
      Frep = prevFrep;
      F = prevF;
      errors = prev_errors;
      queue = queue(2:end, :);
      nextGRnum = prevNextGRnum;
      if DEBUG
	disp('rejected');
      end
    end

    if DEBUG
      GR
    end
  end
  
  return;
