function [grmatrix pairs_conf together] = sfm_seq_groups(tracks, groups, walls, ...
						deltat, peddiam, ...
						pairs)

win_len = 50;
pred_jump = 10;

% SFM parameters
sfmpars.phi = pi/2;
sfmpars.beta1 = 4;
sfmpars.beta2 = 3;
sfmpars.beta3 = 1;
sfmpars.a = 10;
sfmpars.b = 0.1;
sfmpars.tau = 0.5;
sfmpars.A = 4.5;				% 4.5 +- 0.3
sfmpars.lambda = 2.0;			% 2.0 +- 0.2
sfmpars.gamma = 0.35;			% 0.35 +- 0.01
sfmpars.np = 3.0;				% 3.0 +- 0.7
sfmpars.n = 2.0;				% 2.0 +- 0.1
sfmpars.safety_dist = 0; %0.1;
sfmpars.no_move_thr = 0.3;		% threshold on speed

SHOW_FORCES = 0;
SHOW_ERROR = 0;
SHOW_FIGURE = 0;
DEBUG = 0;

N = length(groups);

frames = unique(tracks(:, 4));

win = zeros(N, N, win_len);
win_pos = ones(N, N);

grmatrix = zeros(N, N);

together = zeros(N, N);

first_frames = zeros(1, N);
last_frames = zeros(1, N);
TR = [];
for I=1:N
  TR{I} = tracks(tracks(:, 1) == I, :);
  first_frames(I) = TR{I}(1, 4);
  last_frames(I) = TR{I}(end, 4);
end

pairs_conf = [];
for I=1:length(pairs)
  pairs_conf{I} = [];
end

for t=2:length(frames)-pred_jump+1

%  if DEBUG
    disp(['Frame: ' num2str(t)]);
%  end
  
  tr_idxs = find(first_frames < frames(t-1) & last_frames > frames(t+pred_jump-1));

  if DEBUG
    groups(tr_idxs)
  end
  
  if isempty(tr_idxs)
    continue;
  end

  ped = [];
  xold = [];
  vold = zeros(length(tr_idxs), 2);
  tr_pos = zeros(1, length(tr_idxs));
  for I=1:length(tr_idxs)
    idx = tr_idxs(I);
    tr_pos(I) = find(TR{idx}(:, 4) == frames(t));

    ped(I).d0 = peddiam;
    
    ped(I).x = TR{idx}(tr_pos(I), 2:3);

    vbeg = max(tr_pos(I)-10, 1);
    vlen = tr_pos(I) - vbeg;
    ped(I).v = (TR{idx}(tr_pos(I), 2:3) - TR{idx}(vbeg, 2:3))/(vlen*deltat);
    
    if norm(ped(I).v) < sfmpars.no_move_thr;
      ped(I).e0 = [0 0];
    else
      ped(I).e0 = ped(I).v / norm(ped(I).v);
    end
    ped(I).v0 = norm(ped(I).v);

    xold(I, :) = ped(I).x;
    vold(I, :) = ped(I).v;
  end

  vnew = zeros(length(tr_idxs), 2);
  for I=1:length(tr_idxs)
    idx = tr_idxs(I);

    vbeg = max(tr_pos(I)+pred_jump-10, 1);
    vlen = tr_pos(I) + pred_jump - vbeg + 1;
    vnew(I, :) = (TR{idx}(tr_pos(I)+pred_jump, 2:3) - TR{idx}(vbeg, 2:3))/(vlen*deltat);
  end
  
  GR = zeros(1, length(tr_idxs));
  [GR, grmat] = sfm_find_groups(ped, walls, deltat*pred_jump, vnew, sfmpars);

  if DEBUG
    GR
  
    groups(tr_idxs)
    tr_idxs
  end

  together(tr_idxs, tr_idxs) = together(tr_idxs, tr_idxs) + (grmat > 0);
  
  % window
  win_pos(tr_idxs, tr_idxs) = win_pos(tr_idxs, tr_idxs) + 1;
  win_pos(win_pos > win_len) = 1;

  [idxsA, idxsB] = ndgrid(tr_idxs, tr_idxs);
  idxsC = win_pos(tr_idxs, tr_idxs);
  idxs = sub2ind(size(win), idxsA(:), idxsB(:), idxsC(:));
  win(idxs) = grmat;

  cur_mean = mean(win(tr_idxs, tr_idxs, :), 3);
  grmatrix(tr_idxs, tr_idxs) = max(grmatrix(tr_idxs, tr_idxs), cur_mean);
  
  for I=1:size(pairs, 1)
    pos1 = find(tr_idxs == pairs(I, 1));
    pos2 = find(tr_idxs == pairs(I, 2));
    if ~isempty(pos1) & ~isempty(pos2)
      pairs_conf{I}(end+1) = cur_mean(pos1, pos2);
    end
  end
end